#include <Mysql_Conf.h>
#include <boost/algorithm/string.hpp>
#include <sstream>

#include <ctime>
#include <iomanip>

using namespace inaf::oasbo::Configurators;

MysqlConfigurator::MysqlConfigurator(std::string ip, int port, std::string user,
		std::string password, std::string dbname) {
	try {
		driver = sql::mysql::get_mysql_driver_instance();
		con = driver->connect("tcp://" + ip + ":" + std::to_string(port), user,
				password);
		con->setSchema(dbname);
	} catch (sql::SQLException &e) {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[MYSQL Configurator]\t" << e.what() << std::endl;
	}
}

int MysqlConfigurator::pushConfigToSource(std::string target) {
	if (con == nullptr)
		return -1;

	boost::to_lower(target);
	int result = -1;
	sql::Statement *stmt = nullptr;
	try {
		// Extracts only target entries
		std::map<std::string, std::string> filteredMap;
		for (const auto &pair : this->config) {
			if (pair.first.compare(0, target.length(), target) == 0) {
				std::string key = pair.first.substr(target.length() + 1);
				boost::to_lower(key);
				filteredMap[key] = pair.second;
			}
		}

		std::string tableName = target + "_config";
		stmt = con->createStatement();

		std::stringstream query;
		query << "UPDATE " << tableName << " SET ";
		for (auto conf : filteredMap) {
			query << conf.first << " = '" << conf.second << "',";
		}
		std::string query_str = query.str();
		query_str.replace(query_str.length() - 1, 1,
				" WHERE active = TRUE LIMIT 1;");
		if (stmt->execute(sql::SQLString(query_str)))
			result = 1;

	} catch (sql::SQLException &e) {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[MYSQL Configurator]\t" << e.what() << std::endl;
		result = -1;
	}
	if (stmt)
		delete stmt;

	return result;
}

int MysqlConfigurator::pushConfigToSource() {
	int result = -1;
	if (con == nullptr)
		return result;

	// extract targets
	std::vector<std::string> tagets;

	for (const auto &pair : config) {
		std::string key = pair.first;
		size_t pos = key.find('_');

		if (pos != std::string::npos) {
			std::string prefix = key.substr(0, pos);
			if (std::find(tagets.begin(), tagets.end(), prefix)
					== tagets.end()) {
				tagets.push_back(prefix);
			}
		}
	}
	// call push for each target
	for (auto target : tagets)
		if (pushConfigToSource(target))
			result = 1;
	return result;
}

int MysqlConfigurator::readConfigFromSource(std::string target) {
	int result = -1;
	if (con == nullptr)
		return result;

	boost::to_lower(target);
	std::string tableName = target + "_config";

	sql::Statement *stmt = nullptr;
	sql::ResultSet *res = nullptr;
	try {
		stmt = con->createStatement();
		res = stmt->executeQuery(
				"SELECT * FROM " + tableName + " WHERE active = TRUE "
						+ " LIMIT 1");

		if (res->next()) {
			int numColumns = res->getMetaData()->getColumnCount();
			for (int i = 1; i <= numColumns; i++) {
				std::string columnName = res->getMetaData()->getColumnName(i);
				std::string columnValue = res->getString(i);
				this->config[target + "_" + columnName] = columnValue;
			}
			result = 1;
		} else {
			time_t now = time(nullptr);
			std::cerr << "["
					<< std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
					<< "]\t[MYSQL Configurator]\t" << tableName << " empty"
					<< std::endl;
			result = -1;
		}

	} catch (sql::SQLException &e) {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[MYSQL Configurator]\t" << e.what() << std::endl;
		result = -1;
	}

	if (stmt)
		delete stmt;
	if (res)
		delete res;

	return result;
}

int MysqlConfigurator::readConfigFromSource() {
	if (con == nullptr)
		return -1;

	int result = -1;
	sql::Statement *stmt = nullptr;
	sql::ResultSet *res = nullptr;
	std::vector<std::string> confTables;

	try {
		stmt = con->createStatement();
		res = stmt->executeQuery("SHOW TABLES");

		while (res->next()) {
			std::string tableName = res->getString(1);
			if (tableName.length() >= 5
					&& tableName.substr(tableName.length() - 7) == "_config") {
				confTables.push_back(
						tableName.substr(0, tableName.length() - 7));
			}
		}
		for (auto conftable : confTables) {
			readConfigFromSource(conftable);
			result = 1; // It means that we have read at least 1 conf
		}

	} catch (sql::SQLException &e) {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[MYSQL Configurator]\t" << e.what() << std::endl;
		result = -1;
	}

	if (stmt)
		delete stmt;
	if (res)
		delete res;
	return result;
}

int MysqlConfigurator::insert(std::map<std::string, std::string> conf,
		std::string target) {
	boost::to_lower(target);
	for (const auto &kvp : conf) {
		this->config[target + "_" + kvp.first] = kvp.second;
	}
	return 1;
}

